package org.greenstone.gsdl3.gs3build.util;



public class HTMLDocAnchorList extends HTMLBlockList

{ HTMLDoc doc;

	public HTMLDocAnchorList(HTMLDoc doc)

	{ super();

		this.doc = doc;	// remember the document reference



		super.initialise(doc, anchorStartTags, anchorEndTags, anchorOptEndTags);

	}



	/**
	 *	Return the HTMLBlock containing the named local anchor and it's
	 *	child (contained) tags.
	 *
	 *	@param <code>String</code> name of the local anchor to find.
	 *	@return <code>HTMLBlock</code> the corresponding block of HTML
	 *					contained by the anchor.  If the local anchor does not
	 *					exist, this value will be <code>null</code>.
	 */
	public HTMLBlock localAnchor(String name)

	{	int block;

		String	localname;



		for (block = 0; block < this.size(); block ++)

		{ localname = this.tagBlock(block).headTag().idValue("name");

			if (localname == null)

			{ continue;

			}

			if (name.equalsIgnoreCase(localname))

			{	return this.tagBlock(block);

			}

		}

		return null;

	}

	private int indexOf(String base, String subString, int from)
	{	int i, j;

		for (i = from; i < base.length() - subString.length(); i ++)
		{	if (Character.toLowerCase(base.charAt(i)) == subString.charAt(0))
			{	j = 0;
				for (j = 1; j < subString.length(); j ++)
				{	if (Character.toLowerCase(base.charAt(i+j)) != subString.charAt(j))
					{	break;
					}
				}

				if (j == subString.length())
				{	return i;
				}
			}
		}
		return -1;
	}

	public void flattenLinks()
	{	int 					block;
		String				url;
		char					endsWith;
		StringBuffer	newContent = new StringBuffer(doc.getContent());
		boolean				changed = false;

//		System.out.println("Flattening");

		for (block = 0; block < this.size(); block ++)
		{ HTMLTag tag;
			int			tagPos;
			int			valueStart, valueEnd;

			tag = this.tagBlock(block).headTag();	// get the <A> tag itself

			url = tag.idValue("href"); 						// if there is no HREF member, abandon this one
			if (url == null)
			{ System.out.println("No href");
				continue;
			}
	
			tagPos	= tag.startPos();
			tagPos	=	indexOf(doc.getContent(), "href", tagPos);	// get the HREF itself
			if (tagPos > tag.endPos() || tagPos < 0)					// if none in the tag, skip it
			{	System.out.println("Didn't get href");
				continue;
			}
			tagPos += "href".length();												// skip past it

			while (	doc.getContent().charAt(tagPos) <= ' ' ||		// skip pass equals, etc.
							doc.getContent().charAt(tagPos) == '=')
			{	tagPos ++;
			}

			if (doc.getContent().charAt(tagPos) == '"' ||				// get the demarcating whitespace/quotes
					doc.getContent().charAt(tagPos) == '\'')
			{	endsWith	= doc.getContent().charAt(tagPos);
				tagPos ++;
			}
			else
			{	endsWith  = ' ';
			}
			valueStart = tagPos;

			//			System.out.println("Flattening " + doc.getContent().length() + " " + tagPos);

			while (tagPos < doc.getContent().length() &&
			       doc.getContent().charAt(tagPos) != endsWith)	// read it in
			{	if (doc.content.charAt(tagPos) == '?' ||				// replace special characters			
						doc.content.charAt(tagPos) == '&')
				{	newContent.setCharAt(tagPos, '_');						//   and note that we've done so
					changed	= true;
				}
				tagPos ++;
			}
			valueEnd = tagPos;

			tagPos = indexOf(doc.getContent(), ".html", valueStart);
			if (tagPos >= 0 && tagPos < valueEnd)
			{	// shuffle around the html bit
				while (tagPos < valueEnd - 5)
				{	newContent.setCharAt(tagPos, newContent.charAt(tagPos + 5));
					tagPos ++;
				}
				newContent.setCharAt(tagPos ++, '.');
				newContent.setCharAt(tagPos ++, 'h');
				newContent.setCharAt(tagPos ++, 't');
				newContent.setCharAt(tagPos ++, 'm');
				newContent.setCharAt(tagPos ++, 'l');
			}
		}

		if (changed == true)																// replace the document as needsbe
		{	doc.setContent(newContent.toString());
		}
	}

}
